﻿using OSGeo.FDO.ClientServices;
using OSGeo.FDO.Commands;
using OSGeo.FDO.Commands.DataStore;
using OSGeo.FDO.Commands.Feature;
using OSGeo.FDO.Expression;
using OSGeo.FDO.Geometry;
using System;
using System.Collections.Generic;
using System.Text;

namespace FdoSqlServerBug
{
    class Program
    {
        static void Main(string[] args)
        {
            var connMgr = FeatureAccessManager.GetConnectionManager();
            var conn = connMgr.CreateConnection("OSGeo.SQLServerSpatial");
            conn.ConnectionString = "Service=(local)\\SQLEXPRESS;DataStore=MapGuideTest";

            conn.Open();

            TestCaseTransactional(conn);
            TestCaseNonTransactional(conn);

            conn.Close();

            Console.Read();
        }

        private static void TestCaseTransactional(OSGeo.FDO.Connections.IConnection conn)
        {
            Console.WriteLine("====== TestCaseTransactional ======");
            var trans = conn.BeginTransaction();
            var insertCmd = (IInsert)conn.CreateCommand(CommandType.CommandType_Insert);
            insertCmd.SetFeatureClassName("GeomTest");

            var props = insertCmd.PropertyValues;
            var geomFact = new FgfGeometryFactory();
            var geom = geomFact.CreateGeometry("POINT (3463000 5543000)");
            var fgf = geomFact.GetFgf(geom);

            var geomVal = new GeometryValue(fgf);
            var geomProp = new PropertyValue("Geom", geomVal);
            props.Add(geomProp);

            IFeatureReader reader = null;
            try
            {
                reader = insertCmd.Execute();
                var cls = reader.GetClassDefinition(); //BOOM!
                trans.Commit();
                Console.WriteLine("Success. Transaction committed");
            }
            catch (OSGeo.FDO.Common.Exception ex)
            {
                trans.Rollback();
                Console.WriteLine("You weren't supposed to get here! Reason: {0}", ex.Message);
            }
            finally
            {
                if (reader != null)
                    reader.Close();

                ISelect selectCmd = (ISelect)conn.CreateCommand(CommandType.CommandType_Select);
                selectCmd.SetFeatureClassName("GeomTest");
                int recordCount = 0;
                using (var rdr = selectCmd.Execute())
                {
                    while (rdr.ReadNext()) { recordCount++; }
                    rdr.Close();
                }
                if (recordCount > 0)
                    Console.WriteLine("{0} records in GeomTest", recordCount);
            }
        }

        private static void TestCaseNonTransactional(OSGeo.FDO.Connections.IConnection conn)
        {
            Console.WriteLine("====== TestCaseNonTransactional ======");
            var insertCmd = (IInsert)conn.CreateCommand(CommandType.CommandType_Insert);
            insertCmd.SetFeatureClassName("GeomTest");

            var props = insertCmd.PropertyValues;
            var geomFact = new FgfGeometryFactory();
            var geom = geomFact.CreateGeometry("POINT (3463000 5543000)");
            var fgf = geomFact.GetFgf(geom);

            var geomVal = new GeometryValue(fgf);
            var geomProp = new PropertyValue("Geom", geomVal);
            props.Add(geomProp);

            IFeatureReader reader = null;
            try
            {
                reader = insertCmd.Execute();
                var cls = reader.GetClassDefinition(); //BOOM!
                Console.WriteLine("Success");
            }
            catch (OSGeo.FDO.Common.Exception ex)
            {
                Console.WriteLine("You weren't supposed to get here! Reason: {0}", ex.Message);
            }
            finally //The execute call actually "succeeds" so there is a record in GeomTest
            {
                if (reader != null)
                    reader.Close();

                ISelect selectCmd = (ISelect)conn.CreateCommand(CommandType.CommandType_Select);
                selectCmd.SetFeatureClassName("GeomTest");
                int recordCount = 0;
                using (var rdr = selectCmd.Execute())
                {
                    while (rdr.ReadNext()) { recordCount++; }
                    rdr.Close();
                }
                if (recordCount > 0)
                    Console.WriteLine("{0} records in GeomTest", recordCount);
            }
        }
    }
}
